import React, { useContext, useState, useCallback, useMemo } from "react";
import { Grid, Container, TextField, Autocomplete } from "@mui/material";
import Tabs from "@mui/material/Tabs";
import Tab from "@mui/material/Tab";
import { RootPaper, useAPIMethod } from "@ess-ics/ce-ui-common";
import { initRequestParams } from "../../components/common/Helper";
import { useEffect } from "react";
import { SearchBar } from "../../components/common/SearchBar/SearchBar";
import useUrlState from "@ahooksjs/use-url-state";
import {
  serialize,
  deserialize
} from "../../components/common/URLState/URLState";
import { usePagination } from "../../hooks/pagination";
import { JobTable } from "../../components/Job";
import { apiContext } from "../../api/DeployApi";

export function JobListView() {
  const client = useContext(apiContext);

  const {
    value: operations,
    wrapper: getOperations,
    loading,
    dataReady,
    abort
  } = useAPIMethod({
    fcn: client.apis.Deployments.listOperations,
    call: false
  });

  const [urlState, setUrlState] = useUrlState(
    {
      tab: "0",
      rows: "20",
      page: "0",
      query: "",
      job_type: null
    },
    { navigateMode: "replace" }
  );

  const [deploymentStatus, setDeploymentStatus] = useState(null);

  const handleTabChange = (event, tab) => {
    setUrlState((s) =>
      serialize(s.tab) === serialize(tab)
        ? { tab: serialize(tab) }
        : { tab: serialize(tab), page: "0" }
    );

    changeTab(tab);
  };

  const changeTab = (tab) => {
    if (tab === 0) {
      setDeploymentStatus(null);
    } else if (tab === 1) {
      setDeploymentStatus("ONGOING");
    } else if (tab === 2) {
      setDeploymentStatus("FINISHED");
    }
  };

  useEffect(() => {
    urlState.tab && changeTab(deserialize(urlState.tab));
  }, [urlState]);

  const urlPagination = useMemo(() => {
    return {
      rows: deserialize(urlState.rows),
      page: deserialize(urlState.page)
    };
  }, [urlState.rows, urlState.page]);

  const setUrlPagination = useCallback(
    ({ rows, page }) => {
      setUrlState({
        rows: serialize(rows),
        page: serialize(page)
      });
    },
    [setUrlState]
  );

  const rowsPerPage = [20, 50];

  const { pagination, setPagination } = usePagination({
    rowsPerPageOptions: rowsPerPage,
    initLimit: urlPagination.rows ?? rowsPerPage[0],
    initPage: urlPagination.page ?? 0
  });

  // update pagination whenever search result total pages change
  useEffect(() => {
    setPagination({ totalCount: operations?.totalCount ?? 0 });
  }, [setPagination, operations?.totalCount]);

  // whenever url state changes, update pagination
  useEffect(() => {
    setPagination({ ...urlPagination });
  }, [setPagination, urlPagination]);

  // whenever table pagination internally changes (user clicks next page etc)
  // update the row params
  useEffect(() => {
    setUrlPagination(pagination);
  }, [pagination, setUrlPagination]);

  // Request new search results whenever search or pagination changes
  useEffect(() => {
    let requestParams = initRequestParams(
      urlPagination,
      deserialize(urlState.query)
    );

    if (deploymentStatus) {
      requestParams.status = deploymentStatus;
    }

    if (urlState.job_type) {
      requestParams.type = deserialize(urlState.job_type);
    }

    getOperations(requestParams);

    return () => {
      abort();
    };
  }, [
    getOperations,
    urlPagination,
    urlState.query,
    urlState.job_type,
    deploymentStatus,
    abort
  ]);

  const setSearch = useCallback(
    (query) => {
      setUrlState({ query: serialize(query) });
    },
    [setUrlState]
  );

  // Invoked by Table on change to pagination
  const onPage = (params) => {
    setPagination(params);
    abort();
  };

  const content = (
    <SearchBar
      search={setSearch}
      query={deserialize(urlState.query)}
      loading={loading}
      placeholder="Search"
    >
      <JobTable
        jobs={operations?.operations}
        pagination={pagination}
        onPage={onPage}
        loading={loading || !dataReady}
      />
    </SearchBar>
  );

  const jobTypeOptions = [
    { label: "Show all" },
    { label: "Only deployments", filter: "DEPLOYMENT" },
    { label: "Only commands", filter: "COMMAND" }
  ];

  return (
    <RootPaper>
      <Grid
        container
        spacing={1}
      >
        <Grid
          container
          spacing={1}
          component={Container}
          justifyContent="space-between"
          alignItems="center"
          style={{ display: "flex" }}
        >
          <Grid item>
            <Tabs
              value={deserialize(urlState.tab)}
              onChange={handleTabChange}
            >
              <Tab label="All" />
              <Tab label="Only Ongoing" />
              <Tab label="Only Completed" />
            </Tabs>
          </Grid>
          <Grid
            item
            xs={3}
          >
            <Autocomplete
              id="combo-box-demo"
              options={jobTypeOptions}
              defaultValue={
                urlState.job_type
                  ? jobTypeOptions.find(
                      (o) => o.filter === urlState.job_type
                    ) || {
                      label: "Show all"
                    }
                  : { label: "Show all" }
              }
              getOptionLabel={(option) => option.label}
              isOptionEqualToValue={(option, value) =>
                option.label === value.label
              }
              style={{ width: "100%" }}
              renderInput={(params) => (
                <TextField
                  {...params}
                  label="Job type"
                  variant="outlined"
                />
              )}
              onChange={(event, value, reason) =>
                setUrlState({ job_type: value?.filter })
              }
              autoSelect
            />
          </Grid>
        </Grid>
        <Grid
          item
          xs={12}
          md={12}
        >
          {content}
        </Grid>
      </Grid>
    </RootPaper>
  );
}
